@@ -1,11 +1,12 @@
 /** 
- * XMLSec library
+ * XML Security Library (http://www.aleksey.com/xmlsec).
  *
  * Common XML Doc utility functions
  *
- * See Copyright for the status of this software.
+ * This is free software; see Copyright file in the source
+ * distribution for preciese wording.
  * 
- * Author: Aleksey Sanin <aleksey@aleksey.com>
+ * Copyrigth (C) 2002-2003 Aleksey Sanin <aleksey@aleksey.com>
  */
 #include "globals.h"
 
@@ -15,202 +16,18 @@
 #include <libxml/tree.h>
 #include <libxml/valid.h>
 #include <libxml/xpath.h>
-#include <libxml/parser.h>
 #include <libxml/xpathInternals.h>
-#include <libxml/parserInternals.h>
 
 #include <xmlsec/xmlsec.h>
 #include <xmlsec/xmltree.h>
+#include <xmlsec/parser.h>
 #include <xmlsec/errors.h>
 
-typedef struct _xmlSecExtMemoryParserCtx {
-    const unsigned char 	*prefix;
-    size_t 			prefixSize;
-    const unsigned char 	*buffer;
-    size_t			bufferSize;
-    const unsigned char 	*postfix;
-    size_t 			postfixSize;
-} xmlSecExtMemoryParserCtx, *xmlSecExtMemoryParserCtxPtr;
-
-/** 
- * xmlSecParseFile:
- * @filename: the filename.
- *
- * Loads XML Doc from file @filename. We need a special version because of 
- * c14n issue. The code is copied from xmlSAXParseFileWithData() function.
- *
- * Returns pointer to the loaded XML document or NULL if an error occurs.
- */
-xmlDocPtr
-xmlSecParseFile(const char *filename) {
-    xmlDocPtr ret;
-    xmlParserCtxtPtr ctxt;
-    char *directory = NULL;
-    
-    xmlSecAssert2(filename != NULL, NULL);
-
-    xmlInitParser();
-    ctxt = xmlCreateFileParserCtxt(filename);
-    if (ctxt == NULL) {
-	return(NULL);
-    }
-
-    /* todo: set directories from current doc? */    
-    if ((ctxt->directory == NULL) && (directory == NULL))
-        directory = xmlParserGetDirectory(filename);
-    if ((ctxt->directory == NULL) && (directory != NULL))
-        ctxt->directory = (char *) xmlStrdup((xmlChar *) directory);
-
-    /* required for c14n! */
-    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS; 
-    ctxt->replaceEntities = 1;
-    
-    xmlParseDocument(ctxt);
-
-    if(ctxt->wellFormed) { 
-	ret = ctxt->myDoc;
-    } else {
-       ret = NULL;
-       xmlFreeDoc(ctxt->myDoc);
-       ctxt->myDoc = NULL;
-    }
-    xmlFreeParserCtxt(ctxt);    
-    return(ret);
-    
-}
-
-static int 
-xmlSecExtMemoryParserRead(void * context, char * buffer, int len) {
-    xmlSecExtMemoryParserCtxPtr ctx;
-    size_t size;
-
-    xmlSecAssert2(context != NULL, -1);
-    xmlSecAssert2(buffer != NULL, -1);
-    xmlSecAssert2(len > 0, -1);
-        
-    ctx = (xmlSecExtMemoryParserCtxPtr)context;
-    
-    if((ctx->prefix != NULL) && (ctx->prefixSize > 0)) {
-	size = (ctx->prefixSize < (size_t)len) ? ctx->prefixSize : (size_t)len; 
-	memcpy(buffer, ctx->prefix, size);
-	ctx->prefix += size;
-	ctx->prefixSize -= size;
-	return(size);
-    } else if((ctx->buffer != NULL) && (ctx->bufferSize > 0)) {
-	size = (ctx->bufferSize < (size_t)len) ? ctx->bufferSize : (size_t)len; 
-	memcpy(buffer, ctx->buffer, size);
-	ctx->buffer += size;
-	ctx->bufferSize -= size;
-	return(size);
-    } else if((ctx->postfix != NULL) && (ctx->postfixSize > 0)) {
-	size = (ctx->postfixSize < (size_t)len) ? ctx->postfixSize : (size_t)len; 
-	memcpy(buffer, ctx->postfix, size);
-	ctx->postfix += size;
-	ctx->postfixSize -= size;
-	return(size);
-    }
-    return(0);
-}
-
-/**
- * xmlSecParseMemoryExt:
- * @prefix: the first part of the input.
- * @prefixSize: the size of the first part of the input.
- * @buffer: the second part of the input.
- * @bufferSize: the size of the second part of the input.
- * @postfix: the third part of the input.
- * @postfixSize: the size of the third part of the input.
- *
- * Loads XML Doc from 3 chunks of memory: @prefix, @buffer and @postfix. '
- *
- * Returns pointer to the loaded XML document or NULL if an error occurs.
- */
-xmlDocPtr
-xmlSecParseMemoryExt(const unsigned char *prefix, size_t prefixSize,
-		     const unsigned char *buffer, size_t bufferSize, 
-		     const unsigned char *postfix, size_t postfixSize) {
-    xmlSecExtMemoryParserCtx extCtx;
-    xmlDocPtr ret;
-    xmlParserCtxtPtr ctxt;
-    
-    xmlSecAssert2(buffer != NULL, NULL);
-
-    extCtx.prefix = prefix;
-    extCtx.prefixSize = prefixSize;
-    extCtx.buffer = buffer;
-    extCtx.bufferSize = bufferSize;
-    extCtx.postfix = postfix;
-    extCtx.postfixSize = postfixSize;
-        
-    
-    ctxt = xmlCreateIOParserCtxt(NULL, NULL, xmlSecExtMemoryParserRead, 
-				 NULL, &extCtx, XML_CHAR_ENCODING_NONE);
-    if (ctxt == NULL) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlCreateIOParserCtxt");
-	return(NULL);
-    }
-
-    /* required for c14n! */
-    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS; 
-    ctxt->replaceEntities = 1;
-
-    xmlParseDocument(ctxt);
-    ret = ctxt->myDoc; 
-    xmlFreeParserCtxt(ctxt);    
-    return(ret);
-}
-
-
-/**
- * xmlSecParseMemory:
- * @buffer: the input buffer.
- * @size: the input buffer size.
- * @recovery: the flag.
- *
- * Loads XML Doc from memory. We need a special version because of 
- * c14n issue. The code is copied from xmlSAXParseMemory() function.
- *
- * Returns pointer to the loaded XML document or NULL if an error occurs.
- */
-xmlDocPtr
-xmlSecParseMemory(const unsigned char *buffer, size_t size, int recovery) {
-    xmlDocPtr ret;
-    xmlParserCtxtPtr ctxt;
-
-    xmlSecAssert2(buffer != NULL, NULL);
-    
-    ctxt = xmlCreateMemoryParserCtxt((char*)buffer, size);
-    if (ctxt == NULL) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlCreateMemoryParserCtxt");
-	return(NULL);
-    }
-
-    /* required for c14n! */
-    ctxt->loadsubset = XML_DETECT_IDS | XML_COMPLETE_ATTRS; 
-    ctxt->replaceEntities = 1;
-
-    xmlParseDocument(ctxt);
-
-    if((ctxt->wellFormed) || recovery) {
-	ret = ctxt->myDoc; 
-    } else {
-       ret = NULL;
-       xmlFreeDoc(ctxt->myDoc);
-       ctxt->myDoc = NULL;
-    }
-    xmlFreeParserCtxt(ctxt);    
-    return(ret);
-}
-
 /**
  * xmlSecFindChild:
- * @parent: the pointer to XML node.
- * @name: the name.
- * @ns: the namespace href (may be NULL).
+ * @parent: 		the pointer to XML node.
+ * @name: 		the name.
+ * @ns: 		the namespace href (may be NULL).
  *
  * Searches a direct child of the @parent node having given name and 
  * namespace href.
@@ -239,9 +56,9 @@ xmlSecFindChild(const xmlNodePtr parent, const xmlChar *name, const xmlChar *ns)
 
 /**
  * xmlSecFindParent:
- * @cur: the pointer to an XML node.
- * @name: the name.
- * @ns: the namespace href (may be NULL).
+ * @cur: 		the pointer to an XML node.
+ * @name: 		the name.
+ * @ns: 		the namespace href (may be NULL).
  *
  * Searches the ancestors axis of the @cur node for a node having given name 
  * and namespace href.
@@ -264,9 +81,9 @@ xmlSecFindParent(const xmlNodePtr cur, const xmlChar *name, const xmlChar *ns) {
 
 /**
  * xmlSecFindNode:
- * @parent: the pointer to XML node.
- * @name: the name.
- * @ns: the namespace href (may be NULL).
+ * @parent: 		the pointer to XML node.
+ * @name: 		the name.
+ * @ns: 		the namespace href (may be NULL).
  *
  * Searches all children of the @parent node having given name and 
  * namespace href.
@@ -297,11 +114,39 @@ xmlSecFindNode(const xmlNodePtr parent, const xmlChar *name, const xmlChar *ns)
     return(NULL);
 }
 
+/**
+ * xmlSecGetNodeNsHref:
+ * @cur:		the pointer to node.
+ *
+ * Get's node's namespace href.
+ *
+ * Returns node's namespace href.
+ */
+const xmlChar* 
+xmlSecGetNodeNsHref(const xmlNodePtr cur) {
+    xmlNsPtr ns;
+    
+    xmlSecAssert2(cur != NULL, NULL);
+    
+    /* do we have a namespace in the node? */
+    if(cur->ns != NULL) {
+	return(cur->ns->href);
+    }
+    
+    /* search for default namespace */
+    ns = xmlSearchNs(cur->doc, cur, NULL);
+    if(ns != NULL) {
+	return(ns->href);
+    }
+	
+    return(NULL);
+}
+
 /** 
  * xmlSecCheckNodeName:
- * @cur: the pointer to an XML node.
- * @name: the name,
- * @ns: the namespace href.
+ * @cur: 		the pointer to an XML node.
+ * @name: 		the name,
+ * @ns: 		the namespace href.
  *
  * Checks that the node has a given name and a given namespace href.
  *
@@ -309,29 +154,17 @@ xmlSecFindNode(const xmlNodePtr parent, const xmlChar *name, const xmlChar *ns)
  */
 int
 xmlSecCheckNodeName(const xmlNodePtr cur, const xmlChar *name, const xmlChar *ns) {
+    xmlSecAssert2(cur != NULL, 0);
     
-    if((cur != NULL) && xmlStrEqual(cur->name, name)) {
-	if(cur->ns == NULL && ns == NULL) {
-	    return(1);
-	} else if(cur->ns == NULL) {
-	    xmlNsPtr tmp;
-
-	    tmp = xmlSearchNs(cur->doc, cur, NULL);
-	    if(tmp != NULL && xmlStrEqual(tmp->href, ns)) {
-		return(1);
-	    }
-	} else if(xmlStrEqual(cur->ns->href, ns)){
-	    return(1);	
-	}
-    } 
-    return(0);   
+    return(xmlStrEqual(cur->name, name) && 
+	   xmlStrEqual(xmlSecGetNodeNsHref(cur), ns));
 }
 
 /**
  * xmlSecAddChild:
- * @parent: the pointer to an XML node.
- * @name: the new node name.
- * @ns: the new node namespace.
+ * @parent: 		the pointer to an XML node.
+ * @name: 		the new node name.
+ * @ns: 		the new node namespace.
  *
  * Adds a child to the node @parent with given @name and namespace @ns.
  *
@@ -350,8 +183,10 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
 	text = xmlNewText(BAD_CAST "\n"); 
         if(text == NULL) {	
 	    xmlSecError(XMLSEC_ERRORS_HERE,
+			NULL,
+			"xmlNewText",
 			XMLSEC_ERRORS_R_XML_FAILED,
-			"xmlNewText");
+			XMLSEC_ERRORS_NO_MESSAGE);
 	    return(NULL);
 	}
 	xmlAddChild(parent, text);
@@ -360,8 +195,10 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
     cur = xmlNewChild(parent, NULL, name, NULL);
     if(cur == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlNewChild",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewChild");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(NULL);
     }
 
@@ -372,6 +209,7 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
 	nsPtr = xmlSearchNs(cur->doc, cur, NULL);
 	if((nsPtr == NULL) || !xmlStrEqual(nsPtr->href, ns)) {
 	    nsPtr = xmlNewNs(cur, ns, NULL);
+	    xmlSetNs(cur, nsPtr);
 	}
     }
     
@@ -379,8 +217,10 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
     text = xmlNewText(BAD_CAST "\n"); 
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlNewText",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewText");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(NULL);
     }
     xmlAddChild(parent, text);
@@ -390,9 +230,9 @@ xmlSecAddChild(xmlNodePtr parent, const xmlChar *name, const xmlChar *ns) {
 
 /**
  * xmlSecAddNextSibling
- * @node: the pointer to an XML node.
- * @name: the new node name.
- * @ns: the new node namespace.
+ * @node: 		the pointer to an XML node.
+ * @name: 		the new node name.
+ * @ns: 		the new node namespace.
  *
  * Adds next sibling to the node @node with given @name and namespace @ns.
  *
@@ -409,8 +249,10 @@ xmlSecAddNextSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     cur = xmlNewNode(NULL, name);
     if(cur == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlNewNode",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewNode");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(NULL);
     }
     xmlAddNextSibling(node, cur);
@@ -429,8 +271,10 @@ xmlSecAddNextSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     text = xmlNewText(BAD_CAST "\n");
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlNewText",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewText");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(NULL);
     }
     xmlAddNextSibling(node, text);
@@ -440,9 +284,9 @@ xmlSecAddNextSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
 
 /**
  * xmlSecAddPrevSibling
- * @node: the pointer to an XML node.
- * @name: the new node name.
- * @ns: the new node namespace.
+ * @node: 		the pointer to an XML node.
+ * @name: 		the new node name.
+ * @ns: 		the new node namespace.
  *
  * Adds prev sibling to the node @node with given @name and namespace @ns.
  *
@@ -459,8 +303,10 @@ xmlSecAddPrevSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     cur = xmlNewNode(NULL, name);
     if(cur == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlNewNode",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewNode");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(NULL);
     }
     xmlAddPrevSibling(node, cur);
@@ -479,8 +325,10 @@ xmlSecAddPrevSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
     text = xmlNewText(BAD_CAST "\n");
     if(text == NULL) {	
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlNewText",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewText");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(NULL);
     }
     xmlAddPrevSibling(node, text);
@@ -490,7 +338,7 @@ xmlSecAddPrevSibling(xmlNodePtr node, const xmlChar *name, const xmlChar *ns) {
 
 /**
  * xmlSecGetNextElementNode:
- * @cur: the pointer to an XML node.
+ * @cur: 		the pointer to an XML node.
  *
  * Seraches for the next element node.
  *
@@ -507,8 +355,8 @@ xmlSecGetNextElementNode(xmlNodePtr cur) {
 
 /**
  * xmlSecReplaceNode:
- * @node: the current node.
- * @newNode: the new node.
+ * @node: 		the current node.
+ * @newNode: 		the new node.
  * 
  * Swaps the @node and @newNode in the XML tree.
  *
@@ -516,54 +364,43 @@ xmlSecGetNextElementNode(xmlNodePtr cur) {
  */
 int
 xmlSecReplaceNode(xmlNodePtr node, xmlNodePtr newNode) {
-    xmlNodePtr old;
-    xmlNodePtr ptr;
-    xmlNodePtr dummy;
-
+    xmlNodePtr oldNode;
+    int restoreRoot = 0;
+    
     xmlSecAssert2(node != NULL, -1);
     xmlSecAssert2(newNode != NULL, -1);    
-	    
-    dummy = xmlNewDocNode(newNode->doc, NULL, BAD_CAST "dummy", NULL);
-    if(dummy == NULL) {
+
+    /* fix documents children if necessary first */
+    if((node->doc != NULL) && (node->doc->children == node)) {
+	node->doc->children = node->next;
+	restoreRoot = 1;
+    }
+    if((newNode->doc != NULL) && (newNode->doc->children == newNode)) {
+	newNode->doc->children = newNode->next;
+    }
+
+    oldNode = xmlReplaceNode(node, newNode);
+    if(oldNode == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+                    "xmlReplaceNode",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewDocNode");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(-1);
     }
-	    
-    if(newNode == xmlDocGetRootElement(newNode->doc)) {
-	ptr = xmlDocSetRootElement(newNode->doc, dummy);
-    } else {
-	ptr = xmlReplaceNode(newNode, dummy);
+
+    if(restoreRoot != 0) {
+	xmlDocSetRootElement(oldNode->doc, newNode);
     }
-    if(ptr == NULL) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlDocSetRootElement or xmlReplaceNode");
-	xmlFreeNode(dummy);
-	return(-1);
-    }
-	    
-    if(node == xmlDocGetRootElement(node->doc)) {
-	old = xmlDocSetRootElement(node->doc, ptr);		
-    } else {
-	old = xmlReplaceNode(node, ptr);
-    }
-    if(old == NULL) {
-	xmlSecError(XMLSEC_ERRORS_HERE,
-		    XMLSEC_ERRORS_R_XML_FAILED,
-                    "xmlDocSetRootElement or xmlReplaceNode");
-	xmlFreeNode(ptr);
-	return(-1);
-    }
-    xmlFreeNode(old);
+
+    xmlFreeNode(oldNode);
     return(0);
 }
 
 /**
  * xmlSecReplaceContent
- * @node: the current node.
- * @newNode: the new node.
+ * @node: 		the current node.
+ * @newNode: 		the new node.
  * 
  * Swaps the content of @node and @newNode.
  *
@@ -580,8 +417,10 @@ xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
     dummy = xmlNewDocNode(newNode->doc, NULL, BAD_CAST "dummy", NULL);
     if(dummy == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlNewDocNode",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlNewDocNode");
+		    "node=dummy");
 	return(-1);
     }
 	    
@@ -592,8 +431,10 @@ xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
     }
     if(ptr == NULL) {
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlDocSetRootElement or xmlReplaceNode",
 		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlDocSetRootElement or xmlReplaceNode");
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	xmlFreeNode(dummy);
 	return(-1);
     }
@@ -607,9 +448,9 @@ xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
 
 /**
  * xmlSecReplaceNodeBuffer:
- * @node: the current node.
- * @buffer: the XML data.
- * @size: the XML data size.
+ * @node: 		the current node.
+ * @buffer: 		the XML data.
+ * @size: 		the XML data size.
  * 
  * Swaps the @node and the parsed XML data from the @buffer in the XML tree.
  *
@@ -617,7 +458,7 @@ xmlSecReplaceContent(xmlNodePtr node, xmlNodePtr newNode) {
  */
 int
 xmlSecReplaceNodeBuffer(xmlNodePtr node, 
-			const unsigned char *buffer, size_t size) {
+			const xmlSecByte *buffer, xmlSecSize size) {
     static const char dummyPrefix[] = "<dummy>";
     static const char dummyPostfix[] = "</dummy>";
     xmlDocPtr doc;
@@ -626,19 +467,23 @@ xmlSecReplaceNodeBuffer(xmlNodePtr node,
     xmlSecAssert2(node != NULL, -1);
     xmlSecAssert2(buffer != NULL, -1);    
     
-    doc = xmlSecParseMemoryExt((unsigned char*)dummyPrefix, strlen(dummyPrefix),
+    doc = xmlSecParseMemoryExt((xmlSecByte*)dummyPrefix, strlen(dummyPrefix),
 			       buffer, size,
-			       (unsigned char*)dummyPostfix, strlen(dummyPostfix));
+			       (xmlSecByte*)dummyPostfix, strlen(dummyPostfix));
     if(doc == NULL){
 	xmlSecError(XMLSEC_ERRORS_HERE,
-		    XMLSEC_ERRORS_R_XML_FAILED,
-		    "xmlSecParseMemoryExt");
+		    NULL,
+		    "xmlSecParseMemoryExt",
+		    XMLSEC_ERRORS_R_XMLSEC_FAILED,
+		    XMLSEC_ERRORS_NO_MESSAGE);
 	return(-1);	    	
     }
 	    
     ptr1 = xmlDocGetRootElement(doc);
     if(ptr1 == NULL){
 	xmlSecError(XMLSEC_ERRORS_HERE,
+		    NULL,
+		    "xmlDocGetRootElement",
 		    XMLSEC_ERRORS_R_XML_FAILED,
 		    "root is null");
 	xmlFreeDoc(doc);
@@ -661,9 +506,9 @@ xmlSecReplaceNodeBuffer(xmlNodePtr node,
 
 /**
  * xmlSecAddIDs:
- * @doc: the pointer to an XML document.
- * @cur: the pointer to an XML node.
- * @ids: the pointer to a NULL terminated list of ID attributes.
+ * @doc: 		the pointer to an XML document.
+ * @cur: 		the pointer to an XML node.
+ * @ids: 		the pointer to a NULL terminated list of ID attributes.
  *
  * Walks thru all children of the @cur node and adds all attributes 
  * from the @ids list to the @doc document IDs attributes hash.
@@ -691,8 +536,11 @@ xmlSecAddIDs(xmlDocPtr doc, xmlNodePtr cur, const xmlChar** ids) {
 			    xmlAddID(NULL, doc, name, attr);
 			} else if(tmp != attr) {
 			    xmlSecError(XMLSEC_ERRORS_HERE,
+					NULL,
+					NULL,
 					XMLSEC_ERRORS_R_INVALID_DATA,
-					"id=%s already defined", name);
+					"id=%s already defined", 
+					xmlSecErrorsSafeString(name));
 			}
 			xmlFree(name);
 		    }		    
