@@ -8,9 +8,11 @@ tmpfile=/tmp/testDSig.$timestamp-$$.tmp
 logfile=/tmp/testDSig.$timestamp-$$.log
 script="$0"
 keysfile=$topfolder/keys.xml
+valgrind_suppression="$topfolder/openssl.supp"
+valgrind_options="--leak-check=yes --show-reachable=yes --num-callers=16 -v --suppressions=$valgrind_suppression"
 
 if [ -n "$DEBUG_MEMORY" ] ; then 
-    export VALGRIND="valgrind --leak-check=yes --show-reachable=yes --num-callers=16 -v"
+    export VALGRIND="valgrind $valgrind_options"
     export REPEAT=3
     export EXTRA_PARAMS="--repeat $REPEAT"
 fi
@@ -34,7 +36,9 @@ printRes() {
 execDSigTest() {    
     file=$topfolder/$1      
     echo $1
-    
+
+    rm -f $tmpfile
+        
     printf "    Verify existing signature                            "
     echo "$xmlsec_app verify $2 $file.xml" >> $logfile
     $VALGRIND $xmlsec_app verify $EXTRA_PARAMS $2 $file.xml >> $logfile 2>> $logfile
@@ -137,9 +141,9 @@ execDSigTest "merlin-xmldsig-twenty-three/signature" \
     "--trusted $topfolder/keys/cacert.pem --untrusted $topfolder/keys/ca2cert.pem"
 
 execDSigTest "merlin-xmlenc-five/encsig-ripemd160-hmac-ripemd160-kw-tripledes" \
-    "--keys $topfolder/merlin-xmlenc-five/keys.xml" \
-    "--session-key-hmac --keys $topfolder/merlin-xmlenc-five/keys.xml" \
-    "--keys $topfolder/merlin-xmlenc-five/keys.xml" 
+    "--keys-file $topfolder/merlin-xmlenc-five/keys.xml" \
+    "--session-key hmac-192 --keys-file $topfolder/merlin-xmlenc-five/keys.xml" \
+    "--keys-file $topfolder/merlin-xmlenc-five/keys.xml" 
     
 execDSigTest "merlin-exc-c14n-one/exc-signature" \
     " " \
@@ -147,14 +151,14 @@ execDSigTest "merlin-exc-c14n-one/exc-signature" \
     " " 
     
 execDSigTest "aleksey-xmldsig-01/enveloping-dsa-x509chain" \
-    "--trusted $topfolder/keys/cacert.pem --allowed x509" \
+    "--trusted $topfolder/keys/cacert.pem --enabled-key-data x509" \
     "--privkey $topfolder/keys/dsakey.pem,$topfolder/keys/dsacert.pem,$topfolder/keys/ca2cert.pem" \
-    "--trusted $topfolder/keys/cacert.pem --allowed x509"
+    "--trusted $topfolder/keys/cacert.pem --enabled-key-data x509"
 
 execDSigTest "aleksey-xmldsig-01/enveloping-rsa-x509chain" \
-    "--trusted $topfolder/keys/cacert.pem --allowed x509" \
+    "--trusted $topfolder/keys/cacert.pem --enabled-key-data x509" \
     "--privkey $topfolder/keys/rsakey.pem,$topfolder/keys/rsacert.pem,$topfolder/keys/ca2cert.pem" \
-    "--trusted $topfolder/keys/cacert.pem --allowed x509"
+    "--trusted $topfolder/keys/cacert.pem --enabled-key-data x509"
 
 execDSigTest "aleksey-xmldsig-01/enveloping-hmac-ripemd160" \
     "--hmackey $topfolder/keys/hmackey.bin" \
@@ -182,12 +186,12 @@ execDSigTest "aleksey-xmldsig-01/xpointer-hmac" \
     "--hmackey $topfolder/keys/hmackey.bin" 
 
 execDSigTest "aleksey-xmldsig-01/enveloping-expired-cert" \
-    "--trusted $topfolder/keys/cacert.pem --allowed x509 --verification-time 2002-04-17+10:00:00" 
+    "--trusted $topfolder/keys/cacert.pem --enabled-key-data x509 --verification-time 2002-04-17+10:00:00" 
 
 execDSigTest "aleksey-xmldsig-01/dtd-hmac-91" \
-    "--hmackey $topfolder/keys/hmackey.bin --dtdfile $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
-    "--hmackey $topfolder/keys/hmackey.bin --dtdfile $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
-    "--hmackey $topfolder/keys/hmackey.bin --dtdfile $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd"
+    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" \
+    "--hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd"
 
 execDSigTest "merlin-exc-c14n-one/exc-signature" \
     ""
@@ -195,20 +199,33 @@ execDSigTest "merlin-exc-c14n-one/exc-signature" \
 execDSigTest "merlin-c14n-three/signature" \
     ""
     
-
 execDSigTest "merlin-xpath-filter2-three/sign-xfdl" \
     ""
 
 execDSigTest "merlin-xpath-filter2-three/sign-spec" \
     ""
 
+# test dynamic signature
+echo "Dynamic signature template"
+printf "    Create new signature                                 "
+echo "$xmlsec_app sign-tmpl --keys-file $topfolder/keys.xml --output $tmpfile" >> $logfile
+$VALGRIND $xmlsec_app sign-tmpl $EXTRA_PARAMS --keys-file $topfolder/keys.xml --output $tmpfile >> $logfile 2>> $logfile
+printRes
+printf "    Verify new signature                                 "
+echo "$xmlsec_app verify --keys-file $topfolder/keys.xml $tmpfile" >> $logfile
+$VALGRIND $xmlsec_app verify $EXTRA_PARAMS --keys-file $topfolder/keys.xml $tmpfile >> $logfile 2>> $logfile
+printRes
+
+
 echo "--------- Negative Testing: next test MUST FAIL ----------"
 execDSigTest "merlin-xmldsig-twenty-three/signature-x509-crt-crl" \
     "--trusted $topfolder/merlin-xmldsig-twenty-three/certs/ca.pem"
 
 execDSigTest "aleksey-xmldsig-01/enveloping-expired-cert" \
-    "--trusted $topfolder/keys/cacert.pem --allowed x509" 
+    "--trusted $topfolder/keys/cacert.pem --enabled-key-data x509" 
 
+execDSigTest "aleksey-xmldsig-01/dtd-hmac-91" \
+    "--enabled-reference-uris empty --hmackey $topfolder/keys/hmackey.bin --dtd-file $topfolder/aleksey-xmldsig-01/dtd-hmac-91.dtd" 
     
 rm -rf $tmpfile
 echo "--- testDSig finished" >> $logfile
